#ifndef __BURDEN_H__
#define __BURDEN_H__

#include "FastFit.h"
#include "VcfRecord.h"
#include "VcfFileReader.h"
#include "VcfHeader.h"
#include "SingleVariantTest.h"
#include "GroupFromAnnotation.h"
#include <Eigen/Eigenvalues>


class Burden
{
    public:
        Burden();
        ~Burden();
        static bool vcfAnnotated;
        static bool burden;
        static bool LinScoreAssoc;
        static String genename;
        static String genename_f;
        static double mafMin;
        static double mafMax;
        static double MAC;
        static double miss;
        static String chromosome;
	
	static bool CMC_binary;
	static bool CMC_counts;
	static bool MB;
	static bool SCORE;
	static bool LRT;
	static bool WALD;
	static bool SKAT_BETA;
	static bool SKAT_UW;
	static bool SKAT_MB;
	static bool VT;
	static bool fullResult;

	//Input/Output options	
	static String outputFile;

	VcfFileReader reader;
	VcfHeader header;
	VcfRecord record;


	//this is the total number of genes
	int SNPcount;
	double mac,freq_mean, freq_min, freq_max, freq_se;
	String varsIncluded;
	int numVarPerGene;
	double markerMaf;
	Eigen::VectorXf genotypeMB, genotypeCMC_b, genotypeCMC_c;
	Vector mafIncluded;
	Eigen::VectorXd singlevarPvalue,singlevarEff;
	Eigen::MatrixXf genotypeSKAT,P0;
	Eigen::VectorXf fixedEff, residuals_original;
	double sigma2_hat;

	bool GetGenotypeVectorDosage(FastTransform & trans, Pedigree & ped,GroupFromAnnotation & group,int groupNum);
	bool GetGenotypeVectorVCF(FastTransform & trans, Pedigree & ped,GroupFromAnnotation & group,int groupNum);
	bool GetGenotypeVectorPED(FastTransform & trans, Pedigree & ped,GroupFromAnnotation & group,int groupNum);
	bool GetGenotypeVector(FastTransform & trans, Pedigree & ped,GroupFromAnnotation & group,int groupNum);
	void Calculate(Pedigree & ped, FastTransform & trans,FastFit & engine,String & traitName,GroupFromAnnotation & group,FILE * log);
	void Run(Pedigree & ped, FastTransform & trans, FastFit & engine,String & traitName,GroupFromAnnotation & group,FILE * log);	
	void LinAssoc(FastTransform & trans,FastFit & engine, FILE * out, Eigen::VectorXf & genotype,String & geneName,String & method);
	void MerlinFastAssoc(FastTransform & trans,FILE * out, Eigen::VectorXf & genotype,String & geneName,FastFit & engine,String & method);
	void MerlinLRT(double L0,FastTransform & fullTrans,FILE * out, Eigen::VectorXf & genotype,String & geneName,FastFit & fullEngine,String & method);
	void WaldAssoc(FastTransform & fullTrans,FILE * out, Eigen::VectorXf & genotype,String & geneName,FastFit & fullEngine,String & method);
	void SKATScoreAssoc(FastTransform & trans,FILE * out,String & geneName,FastFit & engine,Pedigree & ped,Eigen::MatrixXf & transGeno, String & method);
	void PrintHeader(FILE * output,FastTransform & trans,FastFit & engine,String & traitName);
	void PrintSKATHeader(FILE * output,FastTransform & trans,FastFit & engine,String & traitName);
	void PrintFooter(FILE * output,int bad,GroupFromAnnotation & group);
	void CalculateLambda(Eigen::VectorXf & weight,double lambda [],Eigen::MatrixXf & transGeno);
	double GetBetaDensity(double a, double b, double x);
	void CalculateSingleVarPvalue(FastTransform & trans,String & geneName,FastFit & engine,Eigen::MatrixXf & genotypeSKAT,Pedigree & ped);
	void AdjustMean(Eigen::VectorXf & genotype,float & mean);
};

#endif
